@@ -35,8 +35,8 @@ class BackgroundServer:
     def line(self, msg, **kw):
         self.tw.line(msg, **kw)
 
-    def _waitup(self, url, count=500):
-        # try for 20 seconds to start devpi-server (which remotely
+    def _waitup(self, url, count=1800):
+        # try to start devpi-server (which remotely
         # receives a serials list which may take a while)
         session = new_requests_session(proxies=False)
         while count > 0:
@@ -59,8 +59,11 @@ class BackgroundServer:
         if not py.path.local(devpi_server).exists():
             self.fatal("not existing devpi-server: %r" % devpi_server)
 
+        url = "http://%s:%s" % (args.host, args.port)
+        if self._waitup(url, count=1):
+            self.fatal("a server is already running at %s" % url)
+
         def prepare_devpiserver(cwd):
-            url = "http://%s:%s" % (args.host, args.port)
             self.line("starting background devpi-server at %s" % url)
             argv = [devpi_server, ] + filtered_args
             #self.line("command: %s" % (argv,))
