@@ -4,15 +4,26 @@ import os.path
 import argparse
 import uuid
 
+from pluggy import PluginManager
 import py
 from devpi_common.types import cached_property
 from .log import threadlog
+from . import hookspecs
 import json
 import devpi_server
 from devpi_common.url import URL
 
 log = threadlog
 
+def get_pluginmanager():
+    pm = PluginManager("devpiserver", implprefix="devpiserver_")
+    pm.add_hookspecs(hookspecs)
+    # XXX load internal plugins here
+    pm.load_setuptools_entrypoints("devpi_server")
+    pm.check_pending()
+    return pm
+
+
 def get_default_serverdir():
     return os.environ.get("DEVPI_SERVERDIR", "~/.devpi/server")
 
@@ -38,6 +49,10 @@ def addoptions(parser):
     web.addoption("--debug", action="store_true",
             help="run wsgi application with debug logging")
 
+    web.addoption("--logger-cfg", action="store", dest="logger_cfg",
+            help="path to yaml/json logger configuration file,"
+                 " requires python2.7 or higher.",
+            default=None)
 
     mirror = parser.addgroup("pypi mirroring options (root/pypi)")
     mirror.addoption("--refresh", type=float, metavar="SECS",
@@ -88,6 +103,11 @@ def addoptions(parser):
                  "This will export all users, indices (except root/pypi),"
                  " release files, test results and documentation. "
     )
+    deploy.addoption("--hard-links", action="store_true",
+            help="use hard links during export instead of copying files. "
+                 "All limitations for hard links on your OS apply. "
+                 "USE AT YOUR OWN RISK"
+    )
     deploy.addoption("--import", type=str, metavar="PATH",
             dest="import_",
             help="import devpi-server database from PATH where PATH "
@@ -97,6 +117,15 @@ def addoptions(parser):
                  "Note that you can only import into a fresh server "
                  "state directory (positional argument to devpi-server).")
 
+    deploy.addoption("--no-events", action="store_false",
+            default=True, dest="wait_for_events",
+            help="no events will be run during import, instead they are"
+                 "postponed to run on server start. This allows much faster "
+                 "start of the server after import, when devpi-web is used. "
+                 "When you start the server after the import, the search "
+                 "index and documentation will gradually update until the "
+                 "server has caught up with all events.")
+
     deploy.addoption("--passwd", action="store", metavar="USER",
             help="set password for user USER (interactive)")
 
@@ -132,19 +161,6 @@ def addoptions(parser):
     #group.addoption("--logfile", action="store",
     #        help="set log file file location")
 
-def load_setuptools_entrypoints():
-    try:
-        from pkg_resources import iter_entry_points, DistributionNotFound
-    except ImportError:
-        return # XXX issue a warning
-    for ep in iter_entry_points('devpi_server'):
-        try:
-            plugin = ep.load()
-        except DistributionNotFound:
-            continue
-        yield plugin, ep.dist
-
-
 def try_argcomplete(parser):
     try:
         import argcomplete
@@ -153,7 +169,7 @@ def try_argcomplete(parser):
     else:
         argcomplete.autocomplete(parser)
 
-def parseoptions(argv, addoptions=addoptions, hook=None):
+def parseoptions(pluginmanager, argv, addoptions=addoptions):
     parser = MyArgumentParser(
         description="Start a server which serves multiples users and "
                     "indices. The special root/pypi index is a real-time "
@@ -161,15 +177,14 @@ def parseoptions(argv, addoptions=addoptions, hook=None):
                     "All indices are suitable for pip or easy_install usage "
                     "and setup.py upload ... invocations."
     )
-
     addoptions(parser)
-    if hook:
-        hook.devpiserver_add_parser_options(parser)
+    pluginmanager.hook.devpiserver_add_parser_options(parser=parser)
+
     try_argcomplete(parser)
     raw = [str(x) for x in argv[1:]]
     args = parser.parse_args(raw)
     args._raw = raw
-    config = Config(args, hook=hook)
+    config = Config(args, pluginmanager=pluginmanager)
     return config
 
 class MyArgumentParser(argparse.ArgumentParser):
@@ -206,72 +221,12 @@ class MyArgumentParser(argparse.ArgumentParser):
 class ConfigurationError(Exception):
     """ incorrect configuration or environment settings. """
 
-class PluginManager:
-    def __init__(self, plugins):
-        self._plugins = plugins
-
-    def _call_plugins(self, _name, **kwargs):
-        results = []
-        for plug, distinfo in self._plugins:
-            meth = getattr(plug, _name, None)
-            if meth is not None:
-                name = "%s.%s" % (
-                            getattr(plug, "__class__", plug).__name__,
-                            meth.__name__)
-                with threadlog.around("debug", "call: %s ", name):
-                    results.append(meth(**kwargs))
-            #else:
-            #    threadlog.error("no plugin hook %s on %s", _name, plug)
-        return results
-
-    def devpiserver_pyramid_configure(self, config, pyramid_config):
-        return self._call_plugins("devpiserver_pyramid_configure",
-                                  config=config, pyramid_config=pyramid_config)
-
-    def devpiserver_add_parser_options(self, parser):
-        return self._call_plugins("devpiserver_add_parser_options",
-                                  parser=parser)
-
-    def devpiserver_run_commands(self, xom):
-        return self._call_plugins("devpiserver_run_commands",
-                                  xom=xom)
-
-    def devpiserver_on_upload(self, stage, projectname, version, link):
-        return self._call_plugins("devpiserver_on_upload",
-                                  stage=stage, projectname=projectname,
-                                  version=version, link=link)
-
-    def devpiserver_on_changed_versiondata(self, stage, projectname, version,
-                                           metadata):
-        return self._call_plugins("devpiserver_on_changed_versiondata",
-                                  stage=stage, projectname=projectname,
-                                  version=version, metadata=metadata)
-
-    def devpiserver_pypi_initial(self, stage, name2serials):
-        return self._call_plugins("devpiserver_pypi_initial",
-                                  stage=stage,
-                                  name2serials=name2serials)
-
-    def devpiserver_auth_user(self, userdict, username, password):
-        """Needs to validate the username and password.
-
-        A dict must be returned with a key "status" with one of the following
-        values:
-            "ok" - authentication succeeded
-            "unknown" - no matching user, other plugins are tried
-            "reject" - invalid password, authentication stops
-
-        Optionally the plugin can return a list of group names the user is
-        member of using the "groups" key of the result dict.
-        """
-        return self._call_plugins("devpiserver_auth_user",
-                                  userdict=userdict,
-                                  username=username, password=password)
-
 
 class Config:
-    def __init__(self, args, hook):
+    def __init__(self, args, pluginmanager):
         self.args = args
+        self.pluginmanager = pluginmanager
+        self.hook = pluginmanager.hook
         serverdir = args.serverdir
         if serverdir is None:
             serverdir = get_default_serverdir()
@@ -284,10 +239,12 @@ class Config:
                     os.path.expanduser(args.secretfile))
 
         self.path_nodeinfo = self.serverdir.join(".nodeinfo")
+
+    def init_nodeinfo(self):
+        log.info("Loading node info from %s", self.path_nodeinfo)
         self._determine_roles()
         self._determine_uuid()
         self.write_nodeinfo()
-        self.hook = hook
 
     def _determine_uuid(self):
         if "uuid" not in self.nodeinfo:
