@@ -38,7 +38,7 @@ class IndexParser:
 
     def _mergelink_ifbetter(self, newurl):
         entry = self.basename2link.get(newurl.basename)
-        if entry is None or (not entry.md5 and newurl.md5):
+        if entry is None or (not entry.hash_spec and newurl.hash_spec):
             self.basename2link[newurl.basename] = newurl
             threadlog.debug("adding link %s", newurl)
         else:
@@ -157,9 +157,9 @@ class PyPIStage(BaseStage):
     username = "root"
     index = "pypi"
     name = "root/pypi"
-    ixconfig = dict(bases=(), volatile=False, type="mirror",
-                    pypi_whitelist=(), custom_data="",
-                    uploadtrigger_jenkins="", acl_upload=["root"])
+    ixconfig = {"bases": (), "volatile": False, "type": "mirror",
+                "pypi_whitelist": (), "custom_data": "",
+                "acl_upload": ["root"]}
 
     def __init__(self, xom):
         self.keyfs = xom.keyfs
@@ -179,7 +179,7 @@ class PyPIStage(BaseStage):
 
     def _dump_project_cache(self, projectname, entries, serial):
         normname = normalize_name(projectname)
-        dumplist = [(entry.relpath, entry.md5, entry.eggfragment)
+        dumplist = [(entry.relpath, entry.hash_spec, entry.eggfragment)
                             for entry in entries]
         data = {"serial": serial,
                 "latest_serial": serial,
@@ -197,13 +197,16 @@ class PyPIStage(BaseStage):
 
     def _load_cache_links(self, projectname):
         cache = self._load_project_cache(projectname)
-        if cache and cache["serial"] >= cache["latest_serial"]:
-            return list(self._make_elinks(projectname, cache["entrylist"]))
+        if cache:
+            return (cache["serial"] >= cache["latest_serial"],
+                   list(self._make_elinks(projectname, cache["entrylist"])))
+        return True, None
 
     def _make_elinks(self, projectname, data):
         from .model import ELink
-        for relpath, md5, eggfragment in data:
-            linkdict = dict(entrypath=relpath, md5=md5, eggfragment=eggfragment)
+        for relpath, hash_spec, eggfragment in data:
+            linkdict = {"entrypath": relpath, "hash_spec": hash_spec,
+                        "eggfragment": eggfragment}
             version = "XXX"
             try:
                 name, version = splitbasename(relpath)[:2]
@@ -230,8 +233,8 @@ class PyPIStage(BaseStage):
         projectname = self.get_projectname_perstage(projectname)
         if projectname is None:
             return []
-        links = self._load_cache_links(projectname)
-        if links is not None:
+        is_fresh, links = self._load_cache_links(projectname)
+        if links is not None and is_fresh:
             return links
 
         # get the simple page for the project
@@ -239,6 +242,11 @@ class PyPIStage(BaseStage):
         threadlog.debug("visiting index %s", url)
         response = self.httpget(url, allow_redirects=True)
         if response.status_code != 200:
+            # if we have an old version, return it instead of erroring out
+            if links is not None:
+                threadlog.error("serving stale links for %r, upstream not reachable",
+                                projectname)
+                return links
             # XXX it's not correct to return UpstreamError in all cases
             # if indeed the project was deleted but that fact
             # is not yet properly processed
@@ -246,12 +254,16 @@ class PyPIStage(BaseStage):
                                      (response.status_code, url))
 
         if self.xom.is_replica():
+            # XXX this code path is not currently tested, handle with care!
+            # we have already triggered the master above
+            # and now need to wait until the parsed new links are
+            # transferred back to the replica
             devpi_serial = int(response.headers["X-DEVPI-SERIAL"])
             self.keyfs.notifier.wait_tx_serial(devpi_serial)
             # XXX raise TransactionRestart to get a consistent clean view
             self.keyfs.commit_transaction_in_thread()
             self.keyfs.begin_transaction_in_thread()
-            links = self._load_cache_links(projectname)
+            is_fresh, links = self._load_cache_links(projectname)
             if links is not None:
                 return links
             raise self.UpstreamError("no cache links from master for %s" %
@@ -315,9 +327,7 @@ class PyPIStage(BaseStage):
                 verdata['name'] = projectname
                 verdata['version'] = version
             links = verdata.setdefault("+elinks", [])
-            links.append(dict(
-                rel="releasefile",
-                entrypath=link.entrypath))
+            links.append({"rel": "releasefile", "entrypath": link.entrypath})
         return verdata
 
 
@@ -359,6 +369,7 @@ class PyPIMirror:
                 fatal("mirror initialization failed: "
                       "pypi.python.org not reachable")
             ensure_unicode_keys(name2serials)
+
             dump_to_file(name2serials, self.path_name2serials)
             # trigger anything (e.g. web-search indexing) that wants to
             # look at the initially loaded serials
@@ -369,11 +380,24 @@ class PyPIMirror:
         return name2serials
 
     def set_project_serial(self, name, serial):
-        """ set the current serial and fill normalization table. """
-        self.name2serials[name] = serial
+        """ set the current serial and update projectname normalization table.
+
+        Usually ``name`` is a "realname" not a normalized name.
+        But you can pass in a normalized name if the project
+        is already known in which case we derive the real name
+        automatically.
+        """
         n = normalize_name(name)
-        if n != name:
-            self.normname2name[n] = name
+        if n in self.normname2name:
+            name = self.normname2name[n]
+
+        if serial is None:
+            del self.name2serials[name]
+            self.normname2name.pop(n, None)
+        else:
+            self.name2serials[name] = serial
+            if n != name:
+                self.normname2name[n] = name
         return n
 
     def thread_run(self, proxy):
