@@ -1,11 +1,14 @@
 from __future__ import unicode_literals
 import sys
-import os
 import json
+import os
 import py
 import logging
+import posixpath
+import shutil
 from devpi_common.validation import normalize_name
-from devpi_common.metadata import splitbasename, BasenameMeta
+from devpi_common.metadata import BasenameMeta
+from devpi_common.types import parse_hash_spec
 from devpi_server.main import fatal
 import devpi_server
 
@@ -36,7 +39,14 @@ def do_import(path, xom):
                   xom.config.serverdir)
     importer = Importer(tw, xom)
     importer.import_all(path)
-    importer.wait_for_events()
+    if xom.config.args.wait_for_events:
+        importer.wait_for_events()
+    else:
+        importer.warn(
+            "Update events have not been processed, when you start the server "
+            "they will be processed in order. If you use devpi-web, then the "
+            "search index and documentation will gradually update until all "
+            "events have been processed.")
     return 0
 
 
@@ -52,12 +62,16 @@ class Exporter:
         self.export_users = self.export["users"] = {}
         self.export_indexes = self.export["indexes"] = {}
 
-    def write_file(self, content, dest):
+    def copy_file(self, src, dest):
         dest.dirpath().ensure(dir=1)
-        with dest.open("wb") as f:
-            f.write(content)
-        self.tw.line("write file at %s" %(dest.relto(self.basepath),))
-        return dest.relto(self.basepath)
+        relpath = dest.relto(self.basepath)
+        if self.config.args.hard_links:
+            self.tw.line("link file at %s" % relpath)
+            os.link(src, dest.strpath)
+        else:
+            self.tw.line("write file at %s" % relpath)
+            shutil.copyfile(src, dest.strpath)
+        return relpath
 
     def warn(self, msg):
         self.tw.line(msg, red=True)
@@ -123,18 +137,17 @@ class IndexDump:
                 linkstore = self.stage.get_linkstore_perstage(vername, version)
                 self.dump_releasefiles(linkstore)
                 self.dump_toxresults(linkstore)
-                content = self.stage.get_doczip(vername, version)
-                if content:
-                    self.dump_docfile(vername, version, content)
+                entry = self.stage.get_doczip_entry(vername, version)
+                if entry:
+                    self.dump_docfile(vername, version, entry)
         self.exporter.completed("index %r" % self.stage.name)
 
     def dump_releasefiles(self, linkstore):
         for link in linkstore.get_links(rel="releasefile"):
             entry = self.exporter.filestore.get_file_entry(link.entrypath)
             assert entry.file_exists(), entry.relpath
-            content = entry.file_get_content()
-            relpath = self.exporter.write_file(
-                content,
+            relpath = self.exporter.copy_file(
+                entry._filepath,
                 self.basedir.join(linkstore.projectname, entry.basename))
             self.add_filedesc("releasefile", linkstore.projectname, relpath,
                                version=linkstore.version,
@@ -144,10 +157,10 @@ class IndexDump:
     def dump_toxresults(self, linkstore):
         for tox_link in linkstore.get_links(rel="toxresult"):
             reflink = linkstore.stage.get_link_from_entrypath(tox_link.for_entrypath)
-            relpath = self.exporter.write_file(
-                content=tox_link.entry.file_get_content(),
-                dest=self.basedir.join(linkstore.projectname, reflink.md5,
-                                       tox_link.basename)
+            relpath = self.exporter.copy_file(
+                tox_link.entry._filepath,
+                self.basedir.join(linkstore.projectname, reflink.hash_spec,
+                                  tox_link.basename)
             )
             self.add_filedesc(type="toxresult",
                               projectname=linkstore.projectname,
@@ -165,11 +178,10 @@ class IndexDump:
         self.indexmeta["files"].append(d)
         self.exporter.completed("%s: %s " %(type, relpath))
 
-    def dump_docfile(self, projectname, version, content):
-        p = self.basedir.join("%s-%s.doc.zip" %(projectname, version))
-        with p.open("wb") as f:
-            f.write(content)
-        relpath = p.relto(self.exporter.basepath)
+    def dump_docfile(self, projectname, version, entry):
+        relpath = self.exporter.copy_file(
+            entry._filepath,
+            self.basedir.join("%s-%s.doc.zip" % (projectname, version)))
         self.add_filedesc("doczip", projectname, relpath, version=version)
 
 
@@ -226,6 +238,11 @@ class Importer:
                     continue
                 import_index = self.import_indexes[stagename]
                 indexconfig = import_index["indexconfig"]
+                if 'uploadtrigger_jenkins' in indexconfig:
+                    if not indexconfig['uploadtrigger_jenkins']:
+                        # remove if not set, so if the trigger was never
+                        # used, you don't need to install the plugin
+                        del indexconfig['uploadtrigger_jenkins']
                 user, index = stagename.split("/")
                 user = self.xom.model.get_user(user)
                 stage = user.create_stage(index, **indexconfig)
@@ -282,16 +299,25 @@ class Importer:
             link = stage.store_releasefile(projectname, version,
                                            p.basename, p.read("rb"),
                                            last_modified=mapping["last_modified"])
-            assert link.entry.md5 == mapping["md5"]
-            self.import_pre2_toxresults(stage, link.entry)
+            # devpi-server-2.1 exported with md5 checksums
+            if "md5" in mapping:
+                assert "hash_spec" not in mapping
+                mapping["hash_spec"] = "md5=" + mapping["md5"]
+            hash_algo, hash_value = parse_hash_spec(mapping["hash_spec"])
+            digest = hash_algo(link.entry.file_get_content()).hexdigest()
+            assert digest == hash_value
+            # note that the actual hash_type used within devpi-server is not
+            # determined here but in store_releasefile/store_doczip/store_toxresult etc
         elif filedesc["type"] == "doczip":
-            basename = os.path.basename(rel)
-            name, version, suffix = splitbasename(basename)
-            link = stage.store_doczip(name, version, p.read("rb"))
+            version = filedesc["version"]
+            link = stage.store_doczip(projectname, version, p.read("rb"))
         elif filedesc["type"] == "toxresult":
             linkstore = stage.get_linkstore_perstage(filedesc["projectname"],
                                            filedesc["version"])
-            link, = linkstore.get_links(entrypath=filedesc["for_entrypath"])
+            # we can not search for the full relative path because
+            # it might use a different checksum
+            basename = posixpath.basename(filedesc["for_entrypath"])
+            link, = linkstore.get_links(basename=basename)
             link = stage.store_toxresult(link, json.loads(p.read("rb").decode("utf8")))
         else:
             fatal("unknown file type: %s" % (type,))
@@ -301,24 +327,6 @@ class Importer:
         else:
             link.add_logs(history_log)
 
-    def import_pre2_toxresults(self, stage, releasefile_entry):
-        # pre 2.0 export structure (called "attachments")
-        md5 = releasefile_entry.md5
-        type_path = self.import_rootdir.join("attach", md5, "toxresult")
-        if not type_path.exists():
-            return
-        releasefile_link = stage.get_link_from_entrypath(
-                entrypath=releasefile_entry.relpath)
-        type = type_path.basename
-        for i in range(len(type_path.listdir())):
-            attachment_data = type_path.join(str(i)).read(mode="rb")
-            attachment_data = attachment_data.decode('utf-8')
-            toxresultdata = json.loads(attachment_data)
-            self.tw.line("importing pre-2.0 test  results %s/%s" %(md5, type))
-            tox_link = stage.store_toxresult(releasefile_link, toxresultdata)
-            tox_link.add_log('upload', '<import>')
-            self.tw.line("imported %s" % tox_link.entrypath)
-
 
 class IndexTree:
     """ sort index inheritance structure to that we can
