@@ -1,14 +1,15 @@
 from __future__ import unicode_literals
 import posixpath
+import sys
 import py
 import re
 import json
 from devpi_common.metadata import sorted_sameproject_links, get_latest_version
 from devpi_common.validation import validate_metadata, normalize_name
-from devpi_common.types import ensure_unicode, cached_property
+from devpi_common.types import ensure_unicode, cached_property, parse_hash_spec
 from time import gmtime
 from .auth import crypt_password, verify_password
-from .filestore import FileEntry, split_md5
+from .filestore import FileEntry
 from .log import threadlog, thread_current_log
 
 
@@ -31,9 +32,19 @@ def run_passwd(root, username):
     user.modify(password=pwd)
 
 
-_ixconfigattr = set((
-    "type", "volatile", "bases", "uploadtrigger_jenkins", "acl_upload",
-    "pypi_whitelist", "custom_data"))
+def get_ixconfigattrs(hooks, index_type):
+    base = set((
+        "type", "volatile", "bases", "acl_upload",
+        "pypi_whitelist", "custom_data"))
+    for defaults in hooks.devpiserver_indexconfig_defaults(index_type=index_type):
+        conflicting = base.intersection(defaults)
+        if conflicting:
+            raise ValueError(
+                "A plugin returned the following keys which conflict with "
+                "existing index configuration keys: %s"
+                % ", ".join(sorted(conflicting)))
+        base.update(defaults)
+    return base
 
 
 class ModelException(Exception):
@@ -63,6 +74,7 @@ class MissesRegistration(ModelException):
 
 class NonVolatile(ModelException):
     """ A release is overwritten on a non volatile index. """
+    link = None  # the conflicting link
 
 
 class RootModel:
@@ -199,8 +211,8 @@ class User:
 
     def create_stage(self, index, type="stage",
                      volatile=True, bases=("root/pypi",),
-                     uploadtrigger_jenkins=None,
-                     acl_upload=None, pypi_whitelist=()):
+                     acl_upload=None, pypi_whitelist=(),
+                     custom_data=None, **kwargs):
         if acl_upload is None:
             acl_upload = [self.name]
         bases = tuple(normalize_bases(self.xom.model, bases))
@@ -211,9 +223,18 @@ class User:
             assert index not in indexes, indexes[index]
             indexes[index] = {
                 "type": type, "volatile": volatile, "bases": bases,
-                "uploadtrigger_jenkins": uploadtrigger_jenkins,
-                "acl_upload": acl_upload, "pypi_whitelist": pypi_whitelist
+                "acl_upload": acl_upload, "pypi_whitelist": pypi_whitelist,
             }
+            if custom_data is not None:
+                indexes[index]["custom_data"] = custom_data
+            hooks = self.xom.config.hook
+            for defaults in hooks.devpiserver_indexconfig_defaults(index_type=type):
+                for key, value in defaults.items():
+                    indexes[index][key] = kwargs.pop(key, value)
+            if kwargs:
+                raise TypeError(
+                    "create_stage() got unexpected keyword arguments: %s"
+                    % ", ".join(kwargs))
         stage = self.getstage(index)
         threadlog.info("created index %s: %s", stage.name, stage.ixconfig)
         return stage
@@ -274,21 +295,21 @@ class BaseStage:
         return l
 
     def list_versions(self, projectname):
-        assert py.builtin._istext(projectname)
+        assert py.builtin._istext(projectname), "projectname %r not text" % projectname
         versions = set()
         for stage, res in self.op_sro_check_pypi_whitelist(
                 "list_versions_perstage", projectname=projectname):
             versions.update(res)
         return versions
 
-    def get_latest_version(self, name):
-        return get_latest_version(self.list_versions(name))
+    def get_latest_version(self, name, stable=False):
+        return get_latest_version(self.list_versions(name), stable=stable)
 
-    def get_latest_version_perstage(self, name):
-        return get_latest_version(self.list_versions_perstage(name))
+    def get_latest_version_perstage(self, name, stable=False):
+        return get_latest_version(self.list_versions_perstage(name), stable=stable)
 
     def get_versiondata(self, projectname, version):
-        assert py.builtin._istext(projectname)
+        assert py.builtin._istext(projectname), "projectname %r not text" % projectname
         result = {}
         for stage, res in self.op_sro_check_pypi_whitelist(
                 "get_versiondata_perstage",
@@ -348,7 +369,7 @@ class BaseStage:
                     if not whitelisted:
                         threadlog.debug("%s: private package %r not whitelisted, "
                                         "ignoring root/pypi", opname, projectname)
-                        break
+                        continue
                     threadlog.debug("private package %r whitelisted at stage %s",
                                     projectname, whitelisted.name)
             else:
@@ -398,12 +419,9 @@ class PrivateStage(BaseStage):
         self.key_projectnames = self.keyfs.PROJNAMES(
                     user=self.user.name, index=self.index)
 
-    def can_upload(self, username):
-        acl_upload = self.ixconfig.get("acl_upload", [])
-        return username in acl_upload or ':ANONYMOUS:' in acl_upload
-
     def modify(self, index=None, **kw):
-        diff = list(set(kw).difference(_ixconfigattr))
+        attrs = get_ixconfigattrs(self.xom.config.hook, self.ixconfig["type"])
+        diff = list(set(kw).difference(attrs))
         if diff:
             raise InvalidIndexconfig(
                 ["invalid keys for index configuration: %s" %(diff,)])
@@ -445,7 +463,7 @@ class PrivateStage(BaseStage):
     def get_projectname_perstage(self, name):
         """ return existing projectname for the given name which may
         be in a non-canonical form. """
-        assert py.builtin._istext(name)
+        assert py.builtin._istext(name), "name %r not text" % name
         names = self.list_projectnames_perstage()
         if name in names:
             return name
@@ -582,14 +600,30 @@ class PrivateStage(BaseStage):
         )
         return link
 
-    def get_doczip(self, name, version):
-        """ get documentation zip as an open file
-        (or None if no docs exists). """
+    def get_doczip_entry(self, name, version):
+        """ get entry of documentation zip or None if no docs exists. """
         linkstore = self.get_linkstore_perstage(name, version)
         links = linkstore.get_links(rel="doczip")
         if links:
-            assert len(links) == 1, links
-            return links[0].entry.file_get_content()
+            if len(links) > 1:
+                def key(link):
+                    when = None
+                    for log in link.get_logs():
+                        when = log.get('when')
+                        if when is not None:
+                            break
+                    return when
+                link = max(links, key=key)
+                threadlog.warn("Multiple documentation files for %s-%s, returning newest", name, version)
+            else:
+                link = links[0]
+            return link.entry
+
+    def get_doczip(self, name, version):
+        """ get documentation zip content or None if no docs exists. """
+        entry = self.get_doczip_entry(name, version)
+        if entry is not None:
+            return entry.file_get_content()
 
 
 class ELink:
@@ -600,6 +634,23 @@ class ELink:
         self.basename = posixpath.basename(self.entrypath)
         self.projectname = projectname
         self.version = version
+        if sys.version_info < (3,0):
+            for key in linkdict:
+                assert py.builtin._istext(key)
+
+    @property
+    def hash_spec(self):
+        return self.linkdict.get("hash_spec", "")
+
+    @property
+    def hash_value(self):
+        return self.hash_spec.split("=")[1]
+
+    def matches_checksum(self, content):
+        hash_algo, hash_value = parse_hash_spec(self.hash_spec)
+        if not hash_algo:
+            return True
+        return hash_algo(content).hexdigest() == hash_value
 
     def __getattr__(self, name):
         try:
@@ -617,7 +668,12 @@ class ELink:
         return self.filestore.get_file_entry(self.entrypath)
 
     def add_log(self, what, who, **kw):
-        self._log.append(dict(what=what, who=who, when=gmtime()[:6], **kw))
+        d = {"what": what, "who": who, "when": gmtime()[:6]}
+        if sys.version_info < (3,0):
+            # make sure keys are unicode as they are on py3
+            kw = dict((py.builtin.text(name), value) for name, value in kw.items())
+        d.update(kw)
+        self._log.append(d)
 
     def add_logs(self, logs):
         self._log.extend(logs)
@@ -645,10 +701,13 @@ class LinkStore:
         overwrite = None
         for link in self.get_links(rel=rel, basename=basename):
             if not self.stage.ixconfig.get("volatile"):
-                raise NonVolatile("rel=%s basename=%s on stage %s" % (
+                exc = NonVolatile("rel=%s basename=%s on stage %s" % (
                     rel, basename, self.stage.name))
+                exc.link = link
+                raise exc
             assert overwrite is None
-            overwrite = sum(x.get('count', 0) for x in link.get_logs() if x.get('what') == 'overwrite')
+            overwrite = sum(x.get('count', 0)
+                            for x in link.get_logs() if x.get('what') == 'overwrite')
             self.remove_links(rel=rel, basename=basename)
         file_entry = self._create_file_entry(basename, file_content)
         if last_modified is not None:
@@ -667,7 +726,7 @@ class LinkStore:
         other_reflinks = self.get_links(rel=rel, for_entrypath=for_entrypath)
         filename = "%s.%s%d" %(base_entry.basename, rel, len(other_reflinks))
         entry = self._create_file_entry(filename, file_content,
-                                        ref_md5=base_entry.md5)
+                                        ref_hash_spec=base_entry.hash_spec)
         return self._add_link_to_file_entry(rel, entry, for_entrypath=for_entrypath)
 
     def remove_links(self, rel=None, basename=None, for_entrypath=None):
@@ -697,16 +756,12 @@ class LinkStore:
         return list(filter(fil, [ELink(self.filestore, linkdict, self.projectname, self.version)
                            for linkdict in self.verdata.get("+elinks", [])]))
 
-    def _create_file_entry(self, basename, file_content, ref_md5=None):
-        if ref_md5 is None:
-            md5dir = None
-        else:
-            md5dir = "/".join(split_md5(ref_md5))
+    def _create_file_entry(self, basename, file_content, ref_hash_spec=None):
         entry = self.filestore.store(
                     user=self.stage.user.name, index=self.stage.index,
                     basename=basename,
                     file_content=file_content,
-                    md5dir=md5dir)
+                    dir_hash_spec=ref_hash_spec)
         entry.projectname = self.projectname
         entry.version = self.version
         return entry
@@ -720,12 +775,11 @@ class LinkStore:
     def _add_link_to_file_entry(self, rel, file_entry, for_entrypath=None):
         if isinstance(for_entrypath, ELink):
             for_entrypath = for_entrypath.entrypath
-        relextra = {}
+        new_linkdict = {"rel": rel, "entrypath": file_entry.relpath,
+                        "hash_spec": file_entry.hash_spec, "_log": []}
         if for_entrypath:
-            relextra["for_entrypath"] = for_entrypath
+            new_linkdict["for_entrypath"] = for_entrypath
         linkdicts = self._get_inplace_linkdicts()
-        new_linkdict = dict(rel=rel, entrypath=file_entry.relpath,
-                            md5=file_entry.md5, _log=[], **relextra)
         linkdicts.append(new_linkdict)
         threadlog.info("added %r link %s", rel, file_entry.relpath)
         self._mark_dirty()
@@ -768,7 +822,7 @@ def add_keys(xom, keyfs):
     keyfs.add_key("PROJVERSION", "{user}/{index}/{name}/{version}/.config", dict)
     keyfs.add_key("PROJNAMES", "{user}/{index}/.projectnames", set)
     keyfs.add_key("STAGEFILE",
-                  "{user}/{index}/+f/{md5a}/{md5b}/{filename}", dict)
+                  "{user}/{index}/+f/{hashdir_a}/{hashdir_b}/{filename}", dict)
 
     sub = EventSubscribers(xom)
     keyfs.notifier.on_key_change("PROJVERSION", sub.on_changed_version_config)
@@ -787,7 +841,8 @@ class EventSubscribers:
         with xom.keyfs.transaction(write=False, at_serial=ev.at_serial):
             stage = xom.model.getstage("root", "pypi")
             name2serials = stage.pypimirror.name2serials
-            hook.devpiserver_pypi_initial(stage, name2serials)
+            hook.devpiserver_pypi_initial(
+                stage=stage, name2serials=name2serials)
 
     def on_changed_version_config(self, ev):
         """ when version config is changed for a project in a stage"""
@@ -813,7 +868,8 @@ class EventSubscribers:
             if metadata != old:
                 stage = self.xom.model.getstage(user, index)
                 hook.devpiserver_on_changed_versiondata(
-                    stage, projectname, version, metadata)
+                    stage=stage, projectname=projectname,
+                    version=version, metadata=metadata)
 
     def on_changed_file_entry(self, ev):
         """ when a file entry is modified. """
@@ -823,14 +879,13 @@ class EventSubscribers:
         keyfs = self.xom.keyfs
         with keyfs.transaction(at_serial=ev.at_serial):
             stage = self.xom.model.getstage(user, index)
-            if stage.ixconfig["type"] == "mirror":
+            if stage is not None and stage.ixconfig["type"] == "mirror":
                 return  # we don't trigger on file changes of pypi mirror
             entry = FileEntry(self.xom, ev.typedkey, meta=ev.value)
-            projectname = stage.get_projectname(entry.projectname)
-            if not projectname or not entry.version:
+            if not entry.projectname or not entry.version:
                 # the entry was deleted
                 return
-            stage = self.xom.model.getstage(user, index)
+            projectname = stage.get_projectname(entry.projectname)
             linkstore = stage.get_linkstore_perstage(
                                                 projectname, entry.version)
             links = linkstore.get_links(basename=entry.basename)
