@@ -1,22 +1,23 @@
 import os
 import py
-import hashlib
 import json
 import contextlib
 import time
-from pyramid.httpexceptions import HTTPNotFound, HTTPAccepted
+from pyramid.httpexceptions import HTTPNotFound, HTTPAccepted, HTTPBadRequest
 from pyramid.view import view_config
 from pyramid.response import Response
+from devpi_common.validation import normalize_name
 from webob.headers import EnvironHeaders, ResponseHeaders
 
 from .keyfs import load, loads, dump, get_write_file_ensure_dir, rename
 from .log import thread_push_log, threadlog
-from .views import is_mutating_http_method, H_MASTER_UUID
+from .views import is_mutating_http_method, H_MASTER_UUID, make_uuid_headers
 from .model import UpstreamError
 
 H_REPLICA_UUID = str("X-DEVPI-REPLICA-UUID")
 H_REPLICA_OUTSIDE_URL = str("X-DEVPI-REPLICA-OUTSIDE-URL")
 H_REPLICA_FILEREPL = str("X-DEVPI-REPLICA-FILEREPL")
+H_EXPECTED_MASTER_ID = str("X-DEVPI-EXPECTED-MASTER-ID")
 
 class MasterChangelogRequest:
     MAX_REPLICA_BLOCK_TIME = 30.0
@@ -34,7 +35,9 @@ class MasterChangelogRequest:
             polling_replicas = self.xom.polling_replicas
             polling_replicas[uuid] = {
                 "remote-ip": self.request.get_remote_ip(),
-                "serial": serial,
+                # the replica always polls its own serial+1
+                # and we want to show where the replica serial is at
+                "serial": int(serial)-1,
                 "in-request": True,
                 "last-request": time.time(),
                 "outside-url": headers.get(H_REPLICA_OUTSIDE_URL),
@@ -60,6 +63,21 @@ class MasterChangelogRequest:
         #   never time out here, leading to more and more threads
         # if no commits happen.
 
+        expected_uuid = self.request.headers.get(H_EXPECTED_MASTER_ID, None)
+        master_uuid = self.xom.config.get_master_uuid()
+        # we require the header but it is allowed to be empty
+        # (during initialization)
+        if expected_uuid is None:
+            msg = "replica sent no %s header" % H_EXPECTED_MASTER_ID
+            threadlog.error(msg)
+            raise HTTPBadRequest(msg)
+
+        if expected_uuid and expected_uuid != master_uuid:
+            threadlog.error("expected %r as master_uuid, replica sent %r", master_uuid,
+                      expected_uuid)
+            raise HTTPBadRequest("expected %s as master_uuid, replica sent %s" %
+                                 (master_uuid, expected_uuid))
+
         serial = self.request.matchdict["serial"]
 
         with self.update_replica_status(serial):
@@ -131,15 +149,17 @@ class ReplicaThread:
         for key in (keyfs.STAGEFILE, keyfs.PYPIFILE_NOMD5):
             keyfs.subscribe_on_import(key, ImportFileReplica(self.xom, errors))
         config = self.xom.config
-        nodeinfo = config.nodeinfo
         while 1:
             self.thread.exit_if_shutdown()
             serial = keyfs.get_next_serial()
             url = self.master_url.joinpath("+changelog", str(serial)).url
             log.info("fetching %s", url)
+            uuid, master_uuid = make_uuid_headers(config.nodeinfo)
+            assert uuid != master_uuid
             try:
                 r = session.get(url, headers={
-                    H_REPLICA_UUID: nodeinfo["uuid"],
+                    H_REPLICA_UUID: uuid,
+                    H_EXPECTED_MASTER_ID: master_uuid,
                     H_REPLICA_OUTSIDE_URL: config.args.outside_url,
                 })
             except session.Errors as e:
@@ -150,17 +170,24 @@ class ReplicaThread:
                 master_uuid = config.get_master_uuid()
                 remote_master_uuid = r.headers.get(H_MASTER_UUID)
                 if not remote_master_uuid:
+                    # we don't fatally leave the process because
+                    # it might just be a temporary misconfiguration
+                    # for example of a nginx frontend
                     log.error("remote provides no %r header, running "
                               "<devpi-server-2.1?"
                               " headers were: %s", H_MASTER_UUID, r.headers)
                     self.thread.sleep(self.ERROR_SLEEP)
                     continue
                 if master_uuid and remote_master_uuid != master_uuid:
-                    log.error("master UUID %r does not match "
-                              "locally recorded master UUID %r",
+                    # we got a master_uuid and it is not the one we
+                    # expect, we are replicating for -- it's unlikely this heals
+                    # itself.  It's thus better to die and signal we can't operate.
+                    log.error("FATAL: master UUID %r does not match "
+                              "expected master UUID %r. EXITTING.",
                               remote_master_uuid, master_uuid)
-                    self.thread.sleep(self.ERROR_SLEEP)
-                    continue
+                    # force exit of the process
+                    os._exit(3)
+
                 if r.status_code == 200:
                     try:
                         changes, rel_renames = loads(r.content)
@@ -178,7 +205,7 @@ class ReplicaThread:
                     log.debug("%s: trying again %s\n", r.status_code, url)
                     continue
                 else:
-                    log.debug("%s: failed fetching %s", r.status_code, url)
+                    log.error("%s: failed fetching %s", r.status_code, url)
             # we got an error, let's wait a bit
             self.thread.sleep(5.0)
 
@@ -210,19 +237,21 @@ class PypiProjectChanged:
         pypimirror = self.xom.pypimirror
         name2serials = pypimirror.name2serials
         cache = ev.value
+
+        # get the normalized projectname (PYPILINKS uses it)
+        nname = ev.typedkey.params["name"]
+        if not nname:
+            threadlog.error("project %r missing", nname)
+            return
+        assert normalize_name(nname) == nname
+
+        projectname = ev.value["projectname"] if cache else nname
         if cache is None:  # deleted
-            # derive projectname to delete from key
-            name = ev.typedkey.params["name"]
-            projectname = pypimirror.get_registered_name(name)
-            if projectname:
-                del name2serials[projectname]
-            else:
-                threadlog.error("project %r missing", name)
+            pypimirror.set_project_serial(projectname, None)
         else:
-            name = cache["projectname"]
-            cur_serial = name2serials.get(name, -1)
+            cur_serial = name2serials.get(projectname, -1)
             if cache and cache["serial"] > cur_serial:
-                name2serials[cache["projectname"]] = cache["serial"]
+                pypimirror.set_project_serial(projectname, cache["serial"])
 
 
 def tween_replica_proxy(handler, registry):
@@ -372,14 +401,13 @@ class ImportFileReplica:
 
         if r.status_code != 200:
             raise FileReplicationError(r)
-        remote_md5 = hashlib.md5(r.content).hexdigest()
-        if entry.md5 and entry.md5 != remote_md5:
+        err = entry.check_checksum(r.content)
+        if err:
             # the file we got is different, it may have changed later.
             # we remember the error and move on
             self.errors.add(dict(
                 url=r.url,
-                message="remote has md5 %s, expected %s" % (
-                    remote_md5, entry.md5),
+                message=str(err),
                 relpath=entry.relpath))
             return
         # in case there were errors before, we can now remove them
