@@ -19,12 +19,11 @@ import json
 from devpi_common.request import new_requests_session
 from devpi_common.validation import normalize_name, is_valid_archive_name
 
-from .model import InvalidIndexconfig, InvalidUser, _ixconfigattr
+from .model import InvalidIndexconfig, InvalidUser, get_ixconfigattrs
 from .keyfs import copy_if_mutable
 from .log import thread_push_log, thread_pop_log, threadlog
 
 from .auth import Auth
-from .config import render_string
 
 server_version = devpi_server.__version__
 
@@ -40,7 +39,12 @@ API_VERSION = "2"
 meta_headers = {str("X-DEVPI-API-VERSION"): str(API_VERSION),
                 str("X-DEVPI-SERVER-VERSION"): server_version}
 
+
 def abort(request, code, body):
+    # if no Accept header is set, then force */*, otherwise the exception
+    # will be returned as text/plain, which causes easy_install/setuptools
+    # to fail improperly
+    request.headers.setdefault("Accept", "*/*")
     if "application/json" in request.headers.get("Accept", ""):
         apireturn(code, body)
     threadlog.error(body)
@@ -321,8 +325,8 @@ class PyPIView:
             href = URL(request.path_info).relpath(href)
             if link.eggfragment:
                 href += "#egg=%s" % link.eggfragment
-            elif link.md5:
-                href += "#md5=%s" % link.md5
+            elif link.hash_spec:
+                href += "#" + link.hash_spec
             links.extend([
                  "/".join(relpath.split("/", 2)[:2]) + " ",
                  html.a(link.basename, href=href),
@@ -392,9 +396,11 @@ class PyPIView:
         route_name="/{user}/{index}/+simple/{name}/refresh", request_method="POST")
     def simple_refresh(self):
         context = self.context
+        # XXX we might want to check if user/index has root/pypi as a base
         stage = context.model.getstage('root', 'pypi')
-        if stage.ixconfig["type"] == "mirror":
-            stage.clear_cache(context.name)
+        assert stage.ixconfig["type"] == "mirror", stage.ixconfig
+        stage.clear_cache(context.name)
+        stage.get_releaselinks_perstage(context.name)
         redirect(self.request.route_url(
             "/{user}/{index}/+simple/{name}",
             user=context.username, index=context.index, name=context.name))
@@ -408,7 +414,7 @@ class PyPIView:
         if not self.request.has_permission("index_create"):
             apireturn(403, "no permission to create index %s/%s" % (
                 self.context.username, self.context.index))
-        kvdict = getkvdict_index(getjson(self.request))
+        kvdict = getkvdict_index(self.xom.config.hook, getjson(self.request))
         try:
             stage = self.context.user.create_stage(self.context.index, **kvdict)
             ixconfig = stage.ixconfig
@@ -423,7 +429,7 @@ class PyPIView:
         stage = self.context.stage
         if stage.name == "root/pypi":
             apireturn(403, "root/pypi index config can not be modified")
-        kvdict = getkvdict_index(getjson(self.request))
+        kvdict = getkvdict_index(self.xom.config.hook, getjson(self.request))
         try:
             ixconfig = stage.modify(**kvdict)
         except InvalidIndexconfig as e:
@@ -478,13 +484,17 @@ class PyPIView:
             auth_user = request.authenticated_userid
             self.log.debug("targetindex %r, auth_user %r", targetindex,
                            auth_user)
-            if not target_stage.can_upload(auth_user):
+            if not request.has_permission("pypi_submit", context=target_stage):
                apireturn(401, message="user %r cannot upload to %r"
                                       %(auth_user, targetindex))
             self._set_versiondata_dict(target_stage, metadata)
             results.append((200, "register", name, version,
                             "->", target_stage.name))
-            results.extend(self._push_links(links, target_stage, name, version))
+            try:
+                results.extend(self._push_links(links, target_stage, name, version))
+            except target_stage.NonVolatile as e:
+                apireturn(409, "%s already exists in non-volatile index" % (
+                          e.link.basename,))
             apireturn(200, result=results, type="actionlog")
         else:
             posturl = pushdata["posturl"]
@@ -620,29 +630,37 @@ class PyPIView:
                     metadata = stage.get_versiondata(projectname, version)
                     if not metadata:
                         abort_submit(400, "could not process form metadata")
+                file_content = content.file.read()
                 try:
                     link = stage.store_releasefile(
                         projectname, version,
-                        content.filename, content.file.read())
-                except stage.NonVolatile:
+                        content.filename, file_content)
+                except stage.NonVolatile as e:
+                    if e.link.matches_checksum(file_content):
+                        abort_submit(200,
+                            "Upload of identical file to non volatile index.")
                     abort_submit(409, "%s already exists in non-volatile index" % (
                          content.filename,))
                 link.add_log(
                     'upload', request.authenticated_userid, dst=stage.name)
-                jenkinurl = stage.ixconfig["uploadtrigger_jenkins"]
-                if jenkinurl:
-                    jenkinurl = jenkinurl.format(pkgname=name,
-                                                 pkgversion=version)
-                    if trigger_jenkins(request, stage, jenkinurl, name) == -1:
-                        abort_submit(200,
-                            "OK, but couldn't trigger jenkins at %s" %
-                            (jenkinurl,))
+                try:
+                    self.xom.config.hook.devpiserver_on_upload_sync(
+                        log=request.log, application_url=request.application_url,
+                        stage=stage, projectname=projectname, version=version)
+                except Exception as e:
+                    abort_submit(200, "OK, but a trigger plugin failed: %s" % e)
             else:
                 doczip = content.file.read()
                 try:
-                    link = stage.store_doczip(name, version, doczip)
+                    link = stage.store_doczip(projectname, version, doczip)
                 except stage.MissesRegistration:
                     apireturn(400, "%s-%s is not registered" %(name, version))
+                except stage.NonVolatile as e:
+                    if e.link.matches_checksum(doczip):
+                        abort_submit(200,
+                            "Upload of identical file to non volatile index.")
+                    abort_submit(409, "%s already exists in non-volatile index" % (
+                         content.filename,))
                 link.add_log(
                     'upload', request.authenticated_userid, dst=stage.name)
         else:
@@ -919,41 +937,6 @@ def getjson(request, allowed_keys=None):
             abort(request, 400, "json keys not recognized: %s" % ",".join(diff))
     return dict
 
-def trigger_jenkins(request, stage, jenkinurl, testspec):
-    log = request.log
-    baseurl = request.application_url
-
-    source = render_string("devpibootstrap.py",
-        INDEXURL=baseurl + "/" + stage.name,
-        VIRTUALENVTARURL= (baseurl +
-            "/root/pypi/+f/f61/cdd983d2c4e6a/"
-            "virtualenv-1.11.6.tar.gz"
-            ),
-        TESTSPEC=testspec,
-        DEVPI_INSTALL_INDEX = baseurl + "/" + stage.name + "/+simple/"
-    )
-    inputfile = py.io.BytesIO(source.encode("ascii"))
-    session = new_requests_session(agent=("server", server_version))
-    try:
-        r = session.post(jenkinurl, data={
-                        "Submit": "Build",
-                        "name": "jobscript.py",
-                        "json": json.dumps(
-                    {"parameter": {"name": "jobscript.py", "file": "file0"}}),
-            },
-                files={"file0": ("file0", inputfile)})
-    except session.Errors:
-        log.error("%s: failed to connect to jenkins at %s",
-                  testspec, jenkinurl)
-        return -1
-
-    if 200 <= r.status_code < 300:
-        log.info("successfully triggered jenkins: %s", jenkinurl)
-    else:
-        log.error("%s: failed to trigger jenkins at %s", r.status_code,
-                  jenkinurl)
-        log.debug(r.content)
-        return -1
 
 def abort_if_invalid_filename(name, filename):
     if not is_valid_archive_name(filename):
@@ -973,9 +956,9 @@ def abort_if_invalid_projectname(request, projectname):
         abort(request, 400, "unicode project names not allowed")
 
 
-def getkvdict_index(req):
+def getkvdict_index(hook, req):
     req_volatile = req.get("volatile")
-    kvdict = dict(volatile=True, type="stage", bases=["root/pypi"])
+    kvdict = {"volatile": True, "type": "stage", "bases": ["root/pypi"]}
     if req_volatile is not None:
         if req_volatile == False or (req_volatile != True and
             req_volatile.lower() in ["false", "no"]):
@@ -986,7 +969,7 @@ def getkvdict_index(req):
             kvdict["bases"] = bases.split(",")
         else:
             kvdict["bases"] = bases
-    additional_keys = _ixconfigattr - set(('volatile', 'bases'))
+    additional_keys = get_ixconfigattrs(hook, "stage") - set(('volatile', 'bases'))
     for key in additional_keys:
         if key in req:
             kvdict[key] = req[key]
