@@ -7,7 +7,7 @@ import mimetypes
 import pytest
 import py
 from bs4 import BeautifulSoup
-from devpi_server.config import PluginManager
+from devpi_server.config import get_pluginmanager
 from devpi_server.main import XOM, parseoptions
 from devpi_common.url import URL
 from devpi_server.extpypi import XMLProxy
@@ -21,6 +21,15 @@ try:
 except ImportError:
     from Queue import Queue as BaseQueue
 
+def make_file_url(basename, content, stagename=None, baseurl="http://localhost/"):
+    from devpi_server.filestore import get_default_hash_spec, make_splitdir
+    hash_spec = get_default_hash_spec(content)
+    hashdir = "/".join(make_splitdir(hash_spec))
+    s = "%s{stage}/+f/%s/%s#%s" %(baseurl, hashdir, basename, hash_spec)
+    if stagename is not None:
+        s = s.format(stage=stagename)
+    return s
+
 class TimeoutQueue(BaseQueue):
     def get(self, timeout=2):
         return BaseQueue.get(self, timeout=timeout)
@@ -55,11 +64,15 @@ def Queue():
 def caplog(caplog):
     """ enrich the pytest-capturelog funcarg. """
     caplog.setLevel(logging.DEBUG)
-    def getrecords(msgrex=None):
+    def getrecords(msgrex=None, minlevel="DEBUG"):
         if msgrex is not None:
             msgrex = re.compile(msgrex)
+        minlevelno = {"DEBUG": 10, "INFO": 20, "WARNING": 30,
+                      "ERROR": 40, "FATAL": 50}.get(minlevel)
         recs = []
         for rec in caplog.records():
+            if rec.levelno < minlevelno:
+                continue
             if msgrex is not None and not msgrex.search(rec.getMessage()):
                 continue
             recs.append(rec)
@@ -134,11 +147,17 @@ def mock():
 @pytest.fixture
 def makexom(request, gentmp, httpget, monkeypatch, mock):
     def makexom(opts=(), httpget=httpget, proxy=None, mocking=True, plugins=()):
-        hook = PluginManager(plugins)
+        from devpi_server import auth_basic
+        from devpi_server import auth_devpi
+        plugins = list(plugins) + [(auth_basic, None), (auth_devpi, None)]
+        pm = get_pluginmanager()
+        for plugin in plugins:
+            pm.register(plugin)
         serverdir = gentmp()
         fullopts = ["devpi-server", "--serverdir", serverdir] + list(opts)
         fullopts = [str(x) for x in fullopts]
-        config = parseoptions(fullopts, hook=hook)
+        config = parseoptions(pm, fullopts)
+        config.init_nodeinfo()
         if mocking:
             if proxy is None:
                 proxy = mock.create_autospec(XMLProxy)
@@ -221,18 +240,22 @@ def httpget(pypiurls):
             log.debug("set mocking response %s %s", mockurl, kw)
             self.url2response[mockurl] = kw
 
-        def mock_simple(self, name, text=None, pkgver=None,
+        def mock_simple(self, name, text=None, pkgver=None, hash_type=None,
             pypiserial=10000, **kw):
             class ret:
-                md5 = None
+                hash_spec = ""
             if pkgver is not None:
                 assert not text
-                if "#" not in pkgver:
-                    ret.md5 = self._getmd5digest(pkgver)
-                    pkgver += "#md5=" + ret.md5
+                if hash_type and "#" not in pkgver:
+                    hv = (pkgver + str(pypiserial)).encode("ascii")
+                    hash_value = getattr(hashlib, hash_type)(hv).hexdigest()
+                    ret.hash_spec = "%s=%s" %(hash_type, hash_value)
+                    pkgver += "#" + ret.hash_spec
                 text = '<a href="../../pkg/{pkgver}" />'.format(pkgver=pkgver)
             elif text and "{md5}" in text:
                 text = text.format(md5=getmd5(text))
+            elif text and "{sha256}" in text:
+                text = text.format(sha256=getsha256(text))
             headers = kw.setdefault("headers", {})
             headers["X-PYPI-LAST-SERIAL"] = pypiserial
             self.mockresponse(pypiurls.simple + name + "/",
@@ -321,6 +344,9 @@ class Mapp(MappMixin):
         commit_serial = int(r.headers["X-DEVPI-SERIAL"])
         self.xom.keyfs.notifier.wait_event_serial(commit_serial)
 
+    def makepkg(self, basename, content, name, version):
+        return content
+
     def delete_user(self, user, code=200):
         r = self.testapp.delete_json("/%s" % user, expect_errors=True)
         assert r.status_code == code
@@ -463,14 +489,15 @@ class Mapp(MappMixin):
         r = self.testapp.patch_json(indexurl, result)
         assert r.status_code == 200
 
-    def set_uploadtrigger_jenkins(self, triggerurl, indexname=None):
+    def set_indexconfig_option(self, key, value, indexname=None):
         indexname = self._getindexname(indexname)
         indexurl = "/" + indexname
         r = self.testapp.get_json(indexurl)
         result = r.json["result"]
-        result["uploadtrigger_jenkins"] = triggerurl
+        result[key] = value
         r = self.testapp.patch_json(indexurl, result)
         assert r.status_code == 200
+        assert r.json["result"][key] == value
 
     def set_pypi_whitelist(self, whitelist, indexname=None):
         indexname = self._getindexname(indexname)
@@ -550,6 +577,18 @@ class Mapp(MappMixin):
         assert r.status_code == code
         if waithooks:
             self._wait_for_serial_in_result(r)
+
+        # return the file url so users/callers can easily use it
+        # (probably the official server response should include the url)
+        r.file_url = make_file_url(basename, content, stagename=indexname)
+        return r
+
+    def push(self, name, version, index, indexname=None, code=200):
+        indexname = self._getindexname(indexname)
+        req = dict(name=name, version=version, targetindex=index)
+        r = self.testapp.push(
+            '/%s' % indexname, py.std.json.dumps(req), expect_errors=True)
+        assert r.status_code == code
         return r
 
     def get_release_paths(self, projectname):
@@ -610,28 +649,42 @@ def noiter(monkeypatch, request):
 class MyTestApp(TApp):
     auth = None
 
+    def __init__(self, *args, **kwargs):
+        super(MyTestApp, self).__init__(*args, **kwargs)
+        self.headers = {}
+
     def set_auth(self, user, password):
         self.auth = (user, password)
 
-    def _gen_request(self, method, url, **kw):
+    def set_header_default(self, name, value):
+        self.headers[str(name)] = str(value)
+
+    def _gen_request(self, method, url, params=None, headers=None, **kw):
+        headers = {} if headers is None else headers.copy()
         if self.auth:
-            headers = kw.get("headers")
             if not headers:
                 headers = kw["headers"] = {}
             headers["X-Devpi-Auth"] = b64encode("%s:%s" % self.auth)
             #print ("setting auth header %r %s %s" % (auth, method, url))
+
+        # fill headers with defaults
+        for name, val in self.headers.items():
+            headers.setdefault(name, val)
+
+        kw["headers"] = headers
+        if params is not None:
+            kw["params"] = params
         return super(MyTestApp, self)._gen_request(method, url, **kw)
 
     def post(self, *args, **kwargs):
         code = kwargs.pop("code", None)
         if code is not None and code >= 300:
             kwargs.setdefault("expect_errors", True)
-        r = super(MyTestApp, self).post(*args, **kwargs)
+        r = self._gen_request("POST", *args, **kwargs)
         if code is not None:
             assert r.status_code == code
         return r
 
-
     def push(self, url, params=None, **kw):
         kw.setdefault("expect_errors", True)
         return self._gen_request("PUSH", url, params=params, **kw)
@@ -642,7 +695,7 @@ class MyTestApp(TApp):
         if accept is not None:
             headers = kwargs.setdefault("headers", {})
             headers[str("Accept")] = str(accept)
-        return super(MyTestApp, self).get(*args, **kwargs)
+        return self._gen_request("GET", *args, **kwargs)
 
     def xget(self, code, *args, **kwargs):
         r = self.get(*args, **kwargs)
@@ -651,7 +704,7 @@ class MyTestApp(TApp):
 
     def xdel(self, code, *args, **kwargs):
         kwargs.setdefault("expect_errors", True)
-        r = self.delete(*args, **kwargs)
+        r = self._gen_request("DELETE", *args, **kwargs)
         assert r.status_code == code
         return r
 
@@ -775,11 +828,16 @@ class Gen:
         if md5 == True:
             self._md5.update(link.encode("utf8"))  # basically random
             link += "#md5=%s" % self._md5.hexdigest()
+        elif md5:
+            link += "#md5=%s" % md5
         return URL(link)
 
 def getmd5(s):
     return hashlib.md5(s.encode("utf8")).hexdigest()
 
+def getsha256(s):
+    return hashlib.sha256(s.encode("utf8")).hexdigest()
+
 
 @pytest.yield_fixture
 def dummyrequest():
