@@ -1,4 +1,5 @@
 from __future__ import unicode_literals
+import hashlib
 import pytest
 
 from devpi_server.extpypi import *
@@ -9,12 +10,23 @@ from test_devpi_server.conftest import getmd5
 class TestIndexParsing:
     simplepy = URL("http://pypi.python.org/simple/py/")
 
-    def test_parse_index_simple(self):
+    @pytest.mark.parametrize("hash_type,hash_value", [
+        ("sha256", "090123"),
+        ("sha224", "1209380123"),
+        ("md5", "102938")
+    ])
+    def test_parse_index_simple_hash_types(self, hash_type, hash_value):
         result = parse_index(self.simplepy,
-            """<a href="../../pkg/py-1.4.12.zip#md5=12ab">qwe</a>""")
+            """<a href="../../pkg/py-1.4.12.zip#%s=%s" /a>"""
+            %(hash_type, hash_value))
         link, = result.releaselinks
         assert link.basename == "py-1.4.12.zip"
-        assert link.md5 == "12ab"
+        assert link.hash_spec == "%s=%s" %(hash_type, hash_value)
+        if hash_type == "md5":
+            assert link.md5 == hash_value
+        else:
+            assert link.md5 is None
+        assert link.hash_algo == getattr(hashlib, hash_type)
 
     def test_parse_index_simple_tilde(self):
         result = parse_index(self.simplepy,
@@ -248,17 +260,12 @@ class TestIndexParsing:
 
 
 class TestExtPYPIDB:
-    def test_parse_project_attributes(self, pypistage):
-        from devpi_server.model import _ixconfigattr
-        for name in _ixconfigattr:
-            assert name in pypistage.ixconfig
-
     def test_parse_project_nomd5(self, pypistage):
-        x = pypistage.mock_simple("pytest", pkgver="pytest-1.0.zip")
+        pypistage.mock_simple("pytest", pkgver="pytest-1.0.zip")
         links = pypistage.get_releaselinks("pytest")
         link, = links
         assert link.entry.url == "https://pypi.python.org/pkg/pytest-1.0.zip"
-        assert link.md5 == x.md5
+        assert not link.hash_spec
         assert link.entrypath.endswith("/pytest-1.0.zip")
         assert link.entrypath == link.entry.relpath
 
@@ -272,16 +279,17 @@ class TestExtPYPIDB:
         links = pypistage.get_releaselinks("pytest")
         assert links[0].eggfragment == "pytest-dev2"
 
-    def test_parse_project_replaced_md5(self, pypistage):
-        x = pypistage.mock_simple("pytest", pypiserial=10,
+    @pytest.mark.parametrize("hash_type", ["md5", "sha256"])
+    def test_parse_project_replaced_md5(self, pypistage, hash_type):
+        x = pypistage.mock_simple("pytest", pypiserial=10, hash_type=hash_type,
                                    pkgver="pytest-1.0.zip")
         links = pypistage.get_releaselinks("pytest")
-        assert links[0].md5 == x.md5
-        y = pypistage.mock_simple("pytest", pypiserial=11,
+        assert links[0].hash_spec == x.hash_spec
+        y = pypistage.mock_simple("pytest", pypiserial=11, hash_type=hash_type,
                                    pkgver="pytest-1.0.zip")
         links = pypistage.get_releaselinks("pytest")
-        assert links[0].md5 == y.md5
-        assert x.md5 != y.md5
+        assert links[0].hash_spec == y.hash_spec
+        assert x.hash_spec != y.hash_spec
 
     def test_get_versiondata_inexistent(self, pypistage):
         pypistage.mock_simple("pytest", status_code=404)
@@ -320,15 +328,16 @@ class TestExtPYPIDB:
         links = pypistage.get_linkstore_perstage("pytest", "1.0").get_links()
         assert len(links) == 1
         assert links[0].basename == "pytest-1.0.zip"
-        assert links[0].entry.md5 == md5
+        assert links[0].entry.hash_spec.startswith("md5=")
+        assert links[0].entry.hash_spec.endswith(md5)
 
         # check refresh
-        md5b = getmd5("456")
+        hashdir_b = getmd5("456")
         pypistage.mock_simple("pytest", text='''
-                <a href="../../pkg/pytest-1.0.1.zip#md5={md5b}" />
+                <a href="../../pkg/pytest-1.0.1.zip#md5={hashdir_b}" />
                 <a href="../../pkg/pytest-1.0.zip#md5={md5}" />
                 <a rel="download" href="https://download.com/index.html" />
-            '''.format(md5=md5, md5b=md5b), pypiserial=25)
+            '''.format(md5=md5, hashdir_b=hashdir_b), pypiserial=25)
         links = pypistage.get_releaselinks("pytest")
         assert len(links) == 3
         assert links[1].entry.url == "https://pypi.python.org/pkg/pytest-1.0.1.zip"
@@ -412,6 +421,21 @@ class TestExtPYPIDB:
         assert pypistage.get_projectname("s_p") == "s-p"
         assert pypistage.get_projectname("sqwe_p") is None
 
+    def test_parse_with_outdated_links_issue165(self, pypistage, caplog):
+        pypistage.mock_simple("pytest", pypiserial=10, pkgver="pytest-1.0.zip")
+        links = pypistage.get_releaselinks("pytest")
+        assert len(links) == 1
+        # update the links just as the PyPIMirror thread would
+        with pypistage.keyfs.PYPILINKS(name="pytest").update() as cache:
+            cache["latest_serial"] = 11
+        # make pypi.python.org unreachable
+        pypistage.mock_simple("pytest", status_code=-1)
+        links2 = pypistage.get_releaselinks("pytest")
+        assert links2[0].linkdict == links[0].linkdict and len(links2) == 1
+        recs = caplog.getrecords("serving stale.*pytest.*")
+        assert len(recs) == 1
+
+
 @pytest.mark.notransaction
 def test_pypi_mirror_redirect_to_canonical_issue139(xom, keyfs, mock):
     proxy = mock.create_autospec(XMLProxy)
@@ -458,7 +482,7 @@ class TestRefreshManager:
         class Plugin:
             def devpiserver_pypi_initial(self, stage, name2serials):
                 queue.put((stage, name2serials))
-        xom = makexom(plugins=[(Plugin(),None)])
+        xom = makexom(plugins=[Plugin()])
         xom.pypimirror.init_pypi_mirror(proxy)
         xom.thread_pool.start()
         stage, name2serials = queue.get()
