@@ -1,4 +1,3 @@
-
 import os
 import pytest
 import py
@@ -10,6 +9,10 @@ zip_types = ("application/zip", "application/x-zip-compressed")
 
 BytesIO = py.io.BytesIO
 
+def getdigest(content, hash_type):
+    return getattr(hashlib, hash_type)(content).hexdigest()
+
+
 class TestFileStore:
     def test_maplink_deterministic(self, filestore, gen):
         link = gen.pypi_package_link("pytest-1.2.zip")
@@ -17,17 +20,22 @@ class TestFileStore:
         entry2 = filestore.maplink(link)
         assert entry1.relpath == entry2.relpath
         assert entry1.basename == "pytest-1.2.zip"
-        assert py.builtin._istext(entry1.md5)
+        assert py.builtin._istext(entry1.hash_spec)
 
-    def test_maplink_splitmd5_issue78(self, filestore, gen):
-        link = gen.pypi_package_link("pytest-1.2.zip")
+    @pytest.mark.parametrize("hash_spec", [
+        "sha256=%s" %(hashlib.sha256(b'qwe').hexdigest()),
+        "md5=%s" %(hashlib.md5(b'qwe').hexdigest()),
+    ])
+    def test_maplink_splithashdir_issue78(self, filestore, gen, hash_spec):
+        link = gen.pypi_package_link("pytest-1.2.zip#" + hash_spec, md5=False)
         entry1 = filestore.maplink(link)
         # check md5 directory structure (issue78)
         parts = entry1.relpath.split("/")
         parent2 = parts[-2]
         parent1 = parts[-3]
-        assert parent1 == link.md5[:3]
-        assert parent2 == link.md5[3:16]
+        assert parent1 == link.hash_value[:3]
+        assert parent2 == link.hash_value[3:16]
+        assert getattr(hashlib, hash_spec.split("=")[0]) == link.hash_algo
 
     def test_maplink(self, filestore, gen):
         link = gen.pypi_package_link("pytest-1.2.zip")
@@ -36,30 +44,54 @@ class TestFileStore:
         assert not entry1.file_exists() and not entry2.file_exists()
         assert entry1 == entry2
         assert entry1.relpath.endswith("/pytest-1.2.zip")
-        assert entry1.md5 == link.md5
+        assert entry1.hash_spec == link.hash_spec
 
     def test_maplink_replaced_release_not_cached_yet(self, filestore, gen):
         link = gen.pypi_package_link("pytest-1.2.zip")
         entry1 = filestore.maplink(link)
         assert not entry1.file_exists()
-        assert entry1.md5 == link.md5
+        assert entry1.hash_spec and entry1.hash_spec == link.hash_spec
         newlink = gen.pypi_package_link("pytest-1.2.zip")
         entry2 = filestore.maplink(newlink)
-        assert entry2.md5 == newlink.md5
+        assert entry2.hash_spec and entry2.hash_spec == newlink.hash_spec
 
     def test_maplink_replaced_release_already_cached(self, filestore, gen):
-        content = b'somedata'
-        md5 = hashlib.md5(content).hexdigest()
-        link = gen.pypi_package_link("pytest-1.2.zip", md5=md5)
-        entry1 = filestore.maplink(link)
-        # pseudo-write a release file
-        entry1.file_set_content(content)
+        content1 = b'somedata'
+        md5_1 = hashlib.md5(content1).hexdigest()
+        link1 = gen.pypi_package_link("pytest-1.2.zip", md5=md5_1)
+        entry1 = filestore.maplink(link1)
+        # pseudo-write a release file with a specific hash_spec
+        entry1.file_set_content(content1, hash_spec="md5=" + md5_1)
         assert entry1.file_exists()
-        newlink = gen.pypi_package_link("pytest-1.2.zip")
-        entry2 = filestore.maplink(newlink)
-        assert entry2.md5 == newlink.md5
+        # make sure the entry has the same hash_spec as the external link
+        assert entry1.hash_spec and entry1.hash_spec == link1.hash_spec
+
+        # now replace the hash of the link and check again
+        content2 = b'otherdata'
+        md5_2 = hashlib.md5(content2).hexdigest()
+        link2 = gen.pypi_package_link("pytest-1.2.zip", md5=md5_2)
+        entry2 = filestore.maplink(link2)
+        assert entry2.hash_spec and entry2.hash_spec == link2.hash_spec
         assert not entry2.file_exists()
 
+    def test_file_exists_new_hash(self, filestore, gen):
+        content1 = b'somedata'
+        md5_1 = hashlib.md5(content1).hexdigest()
+        link1 = gen.pypi_package_link("pytest-1.2.zip", md5=md5_1)
+        entry1 = filestore.maplink(link1)
+        # write a wrong file outside the transaction
+        filepath = entry1._filepath
+        py.path.local(filepath).dirpath().ensure(dir=1)
+        with open(filepath, "w") as f:
+            f.write('othercontent')
+        filestore.keyfs.rollback_transaction_in_thread()
+        filestore.keyfs.begin_transaction_in_thread(write=True)
+        # now check if the file got replaced
+        entry2 = filestore.maplink(link1)
+        assert not entry2.file_exists()
+        filestore.keyfs.commit_transaction_in_thread()
+        assert not py.path.local(filepath).exists()
+
     def test_file_delete(self, filestore, gen):
         link = gen.pypi_package_link("pytest-1.2.zip", md5=False)
         entry1 = filestore.maplink(link)
@@ -76,7 +108,7 @@ class TestFileStore:
         assert not entry1 != entry2
         assert entry1.relpath.endswith("/master")
         assert entry1.eggfragment == "pytest-dev"
-        assert not entry1.md5
+        assert not entry1.hash_spec
         assert entry1.url == link.url_nofrag
         assert entry1.eggfragment == "pytest-dev"
 
@@ -85,26 +117,27 @@ class TestFileStore:
         entry = filestore.maplink(link)
         assert entry.url == link.url
         assert not entry.file_exists()
-        md = hashlib.md5(b"").hexdigest()
-        entry.md5 = md
+        hash_type = "sha256"
+        hash_value = getattr(hashlib, hash_type)(b"").hexdigest()
+        entry.hash_spec = hash_spec = "%s=%s" %(hash_type, hash_value)
         assert not entry.file_exists()
         entry.file_set_content(b"")
         assert entry.file_exists()
         assert entry.url == link.url
-        assert entry.md5 == md
+        assert entry.hash_spec.endswith(hash_value)
 
         # reget
         entry = filestore.get_file_entry(entry.relpath)
         assert entry.file_exists()
         assert entry.url == link.url
-        assert entry.md5 == md
+        assert entry.hash_spec == hash_spec
         entry.delete()
         assert not entry.file_exists()
 
     def test_cache_remote_file(self, filestore, httpget, gen):
         link = gen.pypi_package_link("pytest-1.8.zip", md5=False)
         entry = filestore.maplink(link)
-        assert not entry.md5 and not entry.file_exists()
+        assert not entry.hash_spec and not entry.file_exists()
         filestore.keyfs.restart_as_write_transaction()
         headers={"content-length": "3",
                  "last-modified": "Thu, 25 Nov 2010 20:00:27 GMT",
@@ -123,7 +156,7 @@ class TestFileStore:
         filestore.keyfs.restart_as_write_transaction()
         entry = filestore.get_file_entry(entry.relpath)
         assert entry.file_exists()
-        assert entry.md5 == hashlib.md5(bytes).hexdigest()
+        assert entry.hash_value == getdigest(bytes, entry.hash_type)
         assert entry.file_size() == 3
         rheaders = entry.gethttpheaders()
         assert entry.file_get_content() == b"123"
@@ -153,7 +186,7 @@ class TestFileStore:
     def test_iterfile_remote_no_headers(self, filestore, httpget, gen):
         link = gen.pypi_package_link("pytest-1.8.zip", md5=False)
         entry = filestore.maplink(link)
-        assert not entry.md5
+        assert not entry.hash_spec
         headers={}
         httpget.url2response[link.url] = dict(status_code=200,
                 headers=headers, raw = BytesIO(b"123"))
@@ -166,7 +199,7 @@ class TestFileStore:
     def test_iterfile_remote_error_size_mismatch(self, filestore, httpget, gen):
         link = gen.pypi_package_link("pytest-3.0.zip", md5=False)
         entry = filestore.maplink(link)
-        assert not entry.md5
+        assert not entry.hash_spec
         headers={"content-length": "3",
                  "last-modified": "Thu, 25 Nov 2010 20:00:27 GMT",
                  "content-type": "application/zip"}
@@ -178,7 +211,7 @@ class TestFileStore:
     def test_iterfile_remote_nosize(self, filestore, httpget, gen):
         link = gen.pypi_package_link("pytest-3.0.zip", md5=False)
         entry = filestore.maplink(link)
-        assert not entry.md5
+        assert not entry.hash_spec
         headers={"last-modified": "Thu, 25 Nov 2010 20:00:27 GMT",
                  "content-length": None,}
         assert entry.file_size() is None
@@ -195,7 +228,7 @@ class TestFileStore:
     def test_iterfile_remote_error_md5(self, filestore, httpget, gen):
         link = gen.pypi_package_link("pytest-3.0.zip")
         entry = filestore.maplink(link)
-        assert entry.md5 == link.md5
+        assert entry.hash_spec and entry.hash_spec == link.hash_spec
         headers={"content-length": "3",
                  "last-modified": "Thu, 25 Nov 2010 20:00:27 GMT",
                  "content-type": "application/zip"}
@@ -226,12 +259,12 @@ class TestFileStore:
     def test_store_and_iter(self, filestore):
         content = b"hello"
         entry = filestore.store("user", "index", "something-1.0.zip", content)
-        assert entry.md5 == hashlib.md5(content).hexdigest()
+        assert entry.hash_spec.endswith("="+getdigest(content, entry.hash_type))
         assert entry.file_exists()
         filestore.keyfs.restart_as_write_transaction()
         entry2 = filestore.get_file_entry(entry.relpath)
         assert entry2.basename == "something-1.0.zip"
         assert entry2.file_exists()
-        assert entry2.md5 == entry.md5
+        assert entry2.hash_spec == entry.hash_spec
         assert entry2.last_modified
         assert entry2.file_get_content() == content
