@@ -14,6 +14,12 @@ def test_pkgresources_version_matches_init():
     ver = devpi_server.__version__
     assert pkg_resources.get_distribution("devpi_server").version == ver
 
+def test_version(capfd):
+    main(["devpi-server", "--version"])
+    out, err = capfd.readouterr()
+    assert not err  # not logging output
+    assert devpi_server.__version__ in out.strip()
+
 def test_check_compatible_version_earlier(xom, monkeypatch):
     monkeypatch.setattr(xom, "get_state_version", lambda: "1.0")
     with pytest.raises(Fatal):
@@ -46,7 +52,7 @@ def test_pyramid_configure_called(makexom):
     class Plugin:
         def devpiserver_pyramid_configure(self, config, pyramid_config):
             l.append((config, pyramid_config))
-    xom = makexom(plugins=[(Plugin(),None)])
+    xom = makexom(plugins=[Plugin()])
     xom.create_app()
     assert len(l) == 1
     config, pyramid_config = l[0]
@@ -55,17 +61,19 @@ def test_pyramid_configure_called(makexom):
 
 @wsgi_run_throws
 def test_run_commands_called(monkeypatch, tmpdir):
-    from devpi_server.main import _main
+    from devpi_server.main import _main, get_pluginmanager
     l = []
     class Plugin:
-        def devpiserver_run_commands(self, xom):
+        def devpiserver_cmdline_run(self, xom):
             l.append(xom)
             return 1
     monkeypatch.setattr(devpi_server.extpypi.PyPIMirror, "init_pypi_mirror",
                         lambda self, proxy: None)
+    pm = get_pluginmanager()
+    pm.register(Plugin())
     result = _main(
         argv=["devpi-server", "--serverdir", str(tmpdir)],
-        hook=PluginManager([(Plugin(), None)]))
+        pluginmanager=pm)
     assert result == 1
     assert len(l) == 1
     assert isinstance(l[0], XOM)
@@ -73,16 +81,18 @@ def test_run_commands_called(monkeypatch, tmpdir):
 
 @wsgi_run_throws
 def test_main_starts_server_if_run_commands_returns_none(monkeypatch, tmpdir):
-    from devpi_server.main import _main
+    from devpi_server.main import _main, get_pluginmanager
     l = []
     class Plugin:
-        def devpiserver_run_commands(self, xom):
+        def devpiserver_cmdline_run(self, xom):
             l.append(xom)
     monkeypatch.setattr(devpi_server.extpypi.PyPIMirror, "init_pypi_mirror",
                         lambda self, proxy: None)
+    pm = get_pluginmanager()
+    pm.register(Plugin())
     with pytest.raises(ZeroDivisionError):
         _main(
             argv=["devpi-server", "--serverdir", str(tmpdir)],
-            hook=PluginManager([(Plugin(), None)]))
+            pluginmanager=pm)
     assert len(l) == 1
     assert isinstance(l[0], XOM)
