@@ -17,6 +17,13 @@ def test_view_name2serials(pypistage, testapp):
     entries = load(io)
     assert entries["package"] == 15
 
+@pytest.fixture
+def testapp(testapp):
+    master_uuid = testapp.xom.config.get_master_uuid()
+    assert master_uuid
+    testapp.set_header_default(H_EXPECTED_MASTER_ID, master_uuid)
+    return testapp
+
 
 class TestChangelog:
     replica_uuid = "111"
@@ -69,6 +76,13 @@ class TestChangelog:
         assert r.status_code == 202
         assert int(r.headers["X-DEVPI-SERIAL"]) == latest_serial
 
+    def test_master_id_mismatch(self, testapp):
+        testapp.xget(400, "/+changelog/0", headers={H_EXPECTED_MASTER_ID:str("123")})
+        r = testapp.xget(200, "/+changelog/0", headers={H_EXPECTED_MASTER_ID: ''})
+        assert r.headers[H_MASTER_UUID]
+        del testapp.headers[H_EXPECTED_MASTER_ID]
+        testapp.xget(400, "/+changelog/0")
+
 
 class TestPyPIProxy:
     def test_pypi_proxy(self, xom, reqmock):
@@ -87,24 +101,37 @@ class TestPyPIProxy:
         assert isinstance(replica_xom.proxy, PyPIProxy)
 
 
-def test_pypi_project_changed(replica_xom):
+@pytest.mark.parametrize("normname,realname",
+                         [("proj", "proj"), ("proj-x", "proj_x")])
+def test_pypi_project_changed(replica_xom, normname, realname):
     handler = PypiProjectChanged(replica_xom)
     class Ev:
-        value = dict(projectname="newproject", serial=12)
-        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name="newproject")
+        value = dict(projectname=realname, serial=12)
+        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name=normname)
     handler(Ev())
-    assert replica_xom.pypimirror.name2serials["newproject"] == 12
+    assert replica_xom.pypimirror.name2serials[realname] == 12
+    if normname != realname:
+        assert replica_xom.pypimirror.normname2name[normname] == realname
+
     class Ev2:
-        value = dict(projectname="newproject", serial=15)
-        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name="newproject")
+        value = dict(projectname=realname, serial=15)
+        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name=normname)
     handler(Ev2())
-    assert replica_xom.pypimirror.name2serials["newproject"] == 15
+    assert replica_xom.pypimirror.name2serials[realname] == 15
 
     class Ev3:
-        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name="newproject")
-        value = None
+        value = dict(projectname=realname, serial=12)
+        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name=normname)
     handler(Ev3())
-    assert "newproject" not in replica_xom.pypimirror.name2serials
+    assert replica_xom.pypimirror.name2serials[realname] == 15
+
+    class Ev4:
+        typedkey = replica_xom.keyfs.get_key("PYPILINKS")(name=normname)
+        value = None
+    handler(Ev4())
+    assert realname not in replica_xom.pypimirror.name2serials
+    if normname != realname:
+        assert normname not in replica_xom.pypimirror.normname2name
 
 
 class TestReplicaThread:
@@ -154,7 +181,8 @@ class TestReplicaThread:
             rt.thread_run()
         assert caplog.getrecords("remote.*no.*UUID")
 
-    def test_thread_run_ok_uuid_change(self, rt, mockchangelog, caplog, xom):
+    def test_thread_run_ok_uuid_change(self, rt, mockchangelog, caplog, xom, monkeypatch):
+        monkeypatch.setattr("os._exit", lambda n: 0/0)
         rt.thread.sleep = lambda *x: 0/0
         data = xom.keyfs._fs.get_raw_changelog_entry(0)
         assert data
@@ -350,7 +378,7 @@ class TestFileReplication:
 
         # then we try to return the correct thing
         with xom.keyfs.transaction(write=True):
-            entry.file_set_content(content1, md5=md5)
+            entry.file_set_content(content1)
         xom.httpget.mockresponse(master_file_path, code=200, content=content1)
         replay(xom, replica_xom)
         assert replica_xom.errors.errors == {}
@@ -407,14 +435,14 @@ class TestFileReplication:
         with xom.keyfs.transaction(write=True):
             entry = xom.filestore.maplink(link)
             assert not entry.file_exists()
-            assert not entry.md5
+            assert not entry.hash_spec
 
         replay(xom, replica_xom)
         with replica_xom.keyfs.transaction():
             r_entry = replica_xom.filestore.get_file_entry(entry.relpath)
             assert not r_entry.file_exists()
             assert r_entry.meta
-            assert not r_entry.md5
+            assert not r_entry.hash_spec
 
         with xom.keyfs.transaction(write=True):
             entry.file_set_content(content1)
@@ -445,7 +473,7 @@ class TestFileReplication:
         with xom.keyfs.transaction(write=True):
             link = gen.pypi_package_link("pytest-1.8.zip", md5=True)
             entry = xom.filestore.maplink(link)
-            assert entry.md5 and not entry.file_exists()
+            assert entry.hash_spec and not entry.file_exists()
         replay(xom, replica_xom)
         with replica_xom.keyfs.transaction():
             headers={"content-length": "3",
