@@ -19,12 +19,20 @@ from devpi_server.views import tween_keyfs_transaction, make_uuid_headers
 
 from .functional import TestUserThings, TestIndexThings  # noqa
 
+from devpi_server.filestore import get_default_hash_spec, make_splitdir
+
 proj = pytest.mark.parametrize("proj", [True, False])
 pytestmark = [pytest.mark.notransaction]
 
 def getfirstlink(text):
     return BeautifulSoup(text).findAll("a")[0]
 
+def hash_spec_matches(hash_spec, content):
+    hash_type, hash_value = hash_spec.split("=")
+    digest = getattr(hashlib, hash_type)(content).hexdigest()
+    return digest == hash_value
+
+
 @pytest.mark.parametrize("user,status", [
     ("foo_bar", 'ok'),
     ("foo-bar", 'ok'),
@@ -64,6 +72,8 @@ def test_simple_project(pypistage, testapp):
     r = testapp.get("/root/pypi/+simple/" + name)
     assert r.status_code == 200
     assert r.headers["X-DEVPI-SERIAL"]
+    # easy_install fails if the result isn't html
+    assert "html" in r.headers['content-type']
     assert not BeautifulSoup(r.text).findAll("a")
     path = "/%s-1.0.zip" % name
     pypistage.mock_simple(name, text='<a href="%s"/>' % path)
@@ -84,8 +94,10 @@ def test_simple_project_outside_url_subpath(mapp, outside_url, pypistage, testap
     assert r.status_code == 200
     links = sorted(x["href"] for x in BeautifulSoup(r.text).findAll("a"))
     assert len(links) == 2
+    hash_spec = get_default_hash_spec(b'123')
+    hashdir = "/".join(make_splitdir(hash_spec))
     assert links == [
-        '../+f/202/cb962ac59075b/qpwoei-1.0.tar.gz#md5=202cb962ac59075b964b07152d234b70',
+        '../+f/%s/qpwoei-1.0.tar.gz#%s' % (hashdir, hash_spec),
         '../../../root/pypi/+e/https_pypi.python.org/qpwoei-1.0.zip']
     testapp.xget(
         200, URL("/%s/+simple/qpwoei" % api.stagename).joinpath(links[0]).path,
@@ -149,6 +161,8 @@ def test_simple_list(pypistage, testapp):
     r = testapp.get("/root/pypi/+simple/hello3")
     assert r.status_code == 200
     assert "no such project" in r.text
+    # easy_install fails if the result isn't html
+    assert "html" in r.headers['content-type']
     r = testapp.get("/root/pypi/+simple/")
     assert r.status_code == 200
     links = BeautifulSoup(r.text).findAll("a")
@@ -171,7 +185,7 @@ def test_simple_refresh(mapp, model, pypistage, testapp):
     assert r.location.endswith("/root/pypi/+simple/hello")
     with model.keyfs.transaction(write=False):
         info = pypistage._load_project_cache("hello")
-    assert info == {}
+    assert info["entrylist"] == []
 
 def test_inheritance_versiondata(mapp, model):
     api1 = mapp.create_and_use()
@@ -182,24 +196,38 @@ def test_inheritance_versiondata(mapp, model):
     assert len(r["result"]) == 1
 
 
-def test_simple_refresh_inherited(mapp, model, pypistage, testapp):
-    pypistage.mock_simple("pkg", "<html/>")
-    api = mapp.create_and_use()
-    mapp.set_versiondata(dict(name="pkg", version="1.0"))
-    r = testapp.xget(200, "/%s/+simple/pkg" % api.stagename)
+@pytest.mark.parametrize("projectname", ["pkg", "pkg_some"])
+@pytest.mark.parametrize("stagename", [None, "root/pypi"])
+def test_simple_refresh_inherited(mapp, model, pypistage, testapp, projectname,
+                                  stagename):
+    pypistage.mock_simple(projectname, '<a href="/%s-1.0.zip" />' % projectname,
+                          serial=100)
+    if stagename is None:
+        api = mapp.create_and_use()
+    else:
+        api = mapp.use(stagename)
+    stagename = api.stagename
+
+    r = testapp.xget(200, "/%s/+simple/%s" % (stagename, projectname))
     input, = r.html.select('form input')
     assert input.attrs['name'] == 'refresh'
-    assert input.attrs['value'] == 'Refresh PyPI links'
+    #assert input.attrs['value'] == 'Refresh PyPI links'
     with model.keyfs.transaction(write=False):
-        info = pypistage._load_project_cache("pkg")
+        info = pypistage._load_project_cache(projectname)
     assert info != {}
-    assert info['projectname'] == 'pkg'
-    r = testapp.post("/%s/+simple/pkg/refresh" % api.stagename)
+    assert info['projectname'] == projectname
+    pypistage.mock_simple(projectname, '<a href="/%s-2.0.zip" />' % projectname,
+                          serial=200)
+    r = testapp.post("/%s/+simple/%s/refresh" % (stagename, projectname))
     assert r.status_code == 302
-    assert r.location.endswith("/%s/+simple/pkg" % api.stagename)
+    assert r.location.endswith("/%s/+simple/%s" % (stagename, projectname))
     with model.keyfs.transaction(write=False):
-        info = pypistage._load_project_cache("pkg")
-    assert info == {}
+        info = pypistage._load_project_cache(projectname)
+    assert info["projectname"] == projectname
+    elist = info["entrylist"]
+    assert len(elist) == 1
+    assert elist[0][0].endswith("-2.0.zip")
+
 
 def test_simple_refresh_inherited_not_whitelisted(mapp, testapp):
     api = mapp.create_and_use()
@@ -454,28 +482,65 @@ class TestSubmitValidation:
         submit = submit.__class__(submit.stagename)
         assert not mapp.get_release_paths("pkg5")
 
+    def test_upload_twice_to_nonvolatile(self, submit, testapp, mapp):
+        mapp.modify_index(submit.stagename, indexconfig=dict(volatile=False))
+        metadata = {"name": "Pkg5", "version": "2.6", ":action": "submit"}
+        submit.metadata(metadata, code=200)
+        submit.file("pkg5-2.6.tgz", b"123", {"name": "Pkg5"}, code=200)
+        mapp.upload_doc("pkg5-2.6.doc.zip", b"123", "pkg5", "2.6", code=200)
+        path1, = mapp.get_release_paths("Pkg5")
+        testapp.xget(200, path1)
+        # we now try to upload a different file which should fail
+        r = submit.file("pkg5-2.6.tgz", b"1234", {"name": "Pkg5"}, code=409)
+        assert '409 pkg5-2.6.tgz already exists in non-volatile index' in r.text
+        # import pdb; pdb.set_trace()
+        r = mapp.upload_doc("pkg5-2.6.doc.zip", b"1234", "Pkg5", "2.6", code=409)
+        assert '409 pkg5-2.6.doc.zip already exists in non-volatile index' in r.text
+        # if we upload the same file as originally, then it's a no op
+        r = submit.file("pkg5-2.6.tgz", b"123", {"name": "Pkg5"}, code=200)
+        assert '200 Upload of identical file to non volatile index.' in r.text
+        r = mapp.upload_doc("pkg5-2.6.doc.zip", b"123", "Pkg5", "2.6", code=200)
+        assert '200 Upload of identical file to non volatile index.' in r.text
+        path2, = mapp.get_release_paths("Pkg5")
+        # check that nothing changed
+        assert path1 == path2
+        r = testapp.xget(200, path2)
+        assert r.body == b'123'
+        r = testapp.xget(200, "%s/Pkg5/2.6" % mapp.api.index)
+        links = r.json['result']['+links']
+        assert len(links) == 2
+        for link in links:
+            log1, = link['log']
+            assert sorted(log1.keys()) == ['dst', 'what', 'when', 'who']
+            assert log1['what'] == 'upload'
+            assert log1['who'] == 'user'
+            assert log1['dst'] == 'user/dev'
 
     def test_upload_twice_to_volatile(self, submit, testapp, mapp):
         metadata = {"name": "Pkg5", "version": "2.6", ":action": "submit"}
         submit.metadata(metadata, code=200)
         submit.file("pkg5-2.6.tgz", b"123", {"name": "Pkg5"}, code=200)
+        mapp.upload_doc("pkg5-2.6.doc.zip", b'', "pkg5", "2.6", code=200)
         path1, = mapp.get_release_paths("Pkg5")
         testapp.xget(200, path1)
         submit.file("pkg5-2.6.tgz", b"1234", {"name": "Pkg5"}, code=200)
+        mapp.upload_doc("pkg5-2.6.doc.zip", b"1234", "Pkg5", "2.6", code=200)
         path2, = mapp.get_release_paths("Pkg5")
         testapp.xget(410, path1)  # existed once but deleted during overwrite
         testapp.xget(200, path2)
         r = testapp.xget(200, "%s/Pkg5/2.6" % mapp.api.index)
-        link, = r.json['result']['+links']
-        log1, log2 = link['log']
-        assert sorted(log1.keys()) == ['count', 'what', 'when', 'who']
-        assert log1['what'] == 'overwrite'
-        assert log1['who'] is None
-        assert log1['count'] == 1
-        assert sorted(log2.keys()) == ['dst', 'what', 'when', 'who']
-        assert log2['what'] == 'upload'
-        assert log2['who'] == 'user'
-        assert log2['dst'] == 'user/dev'
+        links = r.json['result']['+links']
+        assert len(links) == 2
+        for link in links:
+            log1, log2 = link['log']
+            assert sorted(log1.keys()) == ['count', 'what', 'when', 'who']
+            assert log1['what'] == 'overwrite'
+            assert log1['who'] is None
+            assert log1['count'] == 1
+            assert sorted(log2.keys()) == ['dst', 'what', 'when', 'who']
+            assert log2['what'] == 'upload'
+            assert log2['who'] == 'user'
+            assert log2['dst'] == 'user/dev'
 
     def test_upload_thrice_and_push(self, submit, testapp, mapp):
         metadata = {"name": "Pkg5", "version": "2.6", ":action": "submit"}
@@ -583,27 +648,32 @@ def test_submit_authorization(mapp, testapp):
 
 
 def test_push_non_existent(mapp, testapp, monkeypatch):
-    # check that push from non-existent index results in 404
     req = dict(name="pkg5", version="2.6", targetindex="user2/dev")
+    # check redirection/404 (depending on if devpi-web is installed,
+    # status_code is different)
     r = testapp.push("/user2/dev/", json.dumps(req), expect_errors=True)
+    assert r.status_code in (302, 404)
+
+    # check that push to from non-existent index results in 404
+    r = testapp.push("/user2/dev", json.dumps(req), expect_errors=True)
     assert r.status_code == 404
     mapp.create_and_login_user("user1", "1")
     mapp.create_index("dev")
 
-    # check that push to non-existent target index results in 404
-    r = testapp.push("/user1/dev/", json.dumps(req), expect_errors=True)
-    assert r.status_code == 404
+    # check that push to non-existent target index results in error
+    r = testapp.push("/user1/dev", json.dumps(req), expect_errors=True)
+    assert r.status_code == 400
 
     mapp.create_and_login_user("user2")
     mapp.create_index("dev", indexconfig=dict(acl_upload=["user2"]))
     mapp.login("user1", "1")
-    # check that push of non-existent release results in 404
-    r = testapp.push("/user1/dev/", json.dumps(req), expect_errors=True)
-    assert r.status_code == 404
+    # check push of non-existent release results in error
+    r = testapp.push("/user1/dev", json.dumps(req), expect_errors=True)
+    assert r.status_code == 400
     #
     mapp.use("user1/dev")
     mapp.upload_file_pypi("pkg5-2.6.tgz", b"123", "pkg5", "2.6")
-    # check that push to non-authoried existent target index results in 401
+    # check that push to non-authorized existent target index results in error
     r = testapp.push("/user1/dev", json.dumps(req), expect_errors=True)
     assert r.status_code == 401
 
@@ -848,7 +918,7 @@ class TestPluginPermissions:
 
     @pytest.fixture
     def xom(self, makexom, plugin):
-        xom = makexom(plugins=[(plugin, None)])
+        xom = makexom(plugins=[plugin])
         return xom
 
     def test_plugin_upload_group(self, mapp, plugin):
@@ -879,15 +949,20 @@ class TestPluginPermissions:
         mapp.create_index("pluginuser/dev", code=404)
 
 
-def test_upload_with_jenkins(mapp, reqmock):
+def test_upload_trigger(mapp):
+    class Plugin:
+        def devpiserver_on_upload_sync(self, log, application_url,
+                                       stage, projectname, version):
+            self.results.append(
+                (application_url, stage.name, projectname, version))
+    plugin = Plugin()
+    plugin.results = []
+    mapp.xom.config.pluginmanager.register(plugin)
     mapp.create_and_use()
-    mapp.set_uploadtrigger_jenkins("http://x.com/{pkgname}/{pkgversion}")
-    rec = reqmock.mockresponse(code=200, url=None)
     mapp.upload_file_pypi("pkg1-2.6.tgz", b"123", "pkg1", "2.6", code=200)
-    assert len(rec.requests) == 1
-    assert rec.requests[0].url == "http://x.com/pkg1/2.6"
-    # XXX properly decode form
-    #assert args[1]["data"]["Submit"] == "Build"
+    assert plugin.results == [
+        ('http://localhost', 'user1/dev', 'pkg1', '2.6')]
+
 
 def test_upload_and_testdata(mapp, testapp):
     from test_devpi_server.example import tox_result_data
@@ -908,13 +983,14 @@ def test_upload_and_testdata(mapp, testapp):
 @proj
 def test_upload_and_access_releasefile_meta(mapp, testapp, proj):
     api = mapp.create_and_use()
-    mapp.upload_file_pypi("pkg5-2.6.tgz", b"123", "pkg5", "2.6")
+    content = b"123"
+    mapp.upload_file_pypi("pkg5-2.6.tgz", content, "pkg5", "2.6")
     vv = get_view_version_links(testapp, api.index, "pkg5", "2.6", proj=proj)
     link = vv.get_link("releasefile")
     pkgmeta = mapp.getjson(link.href)
     assert pkgmeta["type"] == "releasefilemeta"
-    assert pkgmeta["result"]["md5"] == hashlib.md5(b'123').hexdigest()
-
+    hash_spec = pkgmeta["result"]["hash_spec"]
+    assert hash_spec_matches(hash_spec, content)
 
 def test_upload_and_delete_project_version(mapp):
     api = mapp.create_and_use()
@@ -1023,9 +1099,9 @@ def test_wrong_login_format(testapp, mapp):
       dict(type="stage", volatile=False, bases=["root/pypi"],
            acl_upload=["hello"])),
 ])
-def test_kvdict(input, expected):
+def test_kvdict(xom, input, expected):
     from devpi_server.views import getkvdict_index
-    result = getkvdict_index(input)
+    result = getkvdict_index(xom.config.hook, input)
     assert result == expected
 
 
@@ -1153,7 +1229,7 @@ class TestRestrictModify:
 
     @pytest.fixture
     def xom(self, makexom, plugin, restrict_modify):
-        xom = makexom(plugins=[(plugin, None)])
+        xom = makexom(plugins=[plugin])
         xom.config.args.restrict_modify = restrict_modify
         return xom
 
